@@ -9,6 +9,7 @@ require "vagrant/util/platform"
 require "vagrant/util/subprocess"
 require "vagrant/util/curl_helper"
 require "vagrant/util/file_checksum"
+require "vagrant/util/map_command_options"
 
 module Vagrant
   module Util
@@ -69,6 +70,7 @@ module Vagrant
           :sha384 => options[:sha384],
           :sha512 => options[:sha512]
         }.compact
+        @extra_download_options = options[:box_extra_download_options] || []
       end
 
       # This executes the actual download, downloading the source file
@@ -183,6 +185,7 @@ module Vagrant
 
       def execute_curl(options, subprocess_options, &data_proc)
         options = options.dup
+        options.unshift("-q")
         options << subprocess_options
 
         # Create the callback that is called if we are interrupted
@@ -230,7 +233,6 @@ module Vagrant
       def options
         # Build the list of parameters to execute with cURL
         options = [
-          "-q",
           "--fail",
           "--location",
           "--max-redirs", "10", "--verbose",
@@ -245,6 +247,8 @@ module Vagrant
         options << "-u" << @auth if @auth
         options << "--location-trusted" if @location_trusted
 
+        options.concat(@extra_download_options)
+
         if @headers
           Array(@headers).each do |header|
             options << "-H" << header
