@@ -5,8 +5,8 @@ module VagrantPlugins
         def self.change_host_name(machine, name)
           comm = machine.communicate
 
-          if !comm.test("getent hosts '#{name}'", sudo: false)
-            basename = name.split(".", 2)[0]
+          basename = name.split(".", 2)[0]
+          if !comm.test('test "$(hostnamectl --static status)" = "#{basename}"', sudo: false)
             comm.sudo <<-EOH.gsub(/^ {14}/, '')
               hostnamectl set-hostname '#{basename}'
 
