@@ -16,6 +16,28 @@ module VagrantPlugins
           @uuid = uuid
         end
 
+        # Controller-Port-Device looks like:
+        # SATA Controller-ImageUUID-0-0 (sub out ImageUUID)
+        # - Controller: SATA Controller
+        # - Port: 0
+        # - Device: 0
+        #
+        # @param [String] port - port on device to attach disk to
+        # @param [String] device - device on controller for disk
+        # @param [String] file - disk file path
+        # @param [String] type - type of disk to attach
+        # @param [Hash]   opts -  additional options
+        def attach_disk(port, device, file, type="hdd", **opts)
+          # Maybe only support SATA Controller for `:disk`???
+          controller = "SATA Controller"
+
+          comment = "This disk is managed externally by Vagrant. Removing or adjusting settings could potentially cause issues with Vagrant."
+
+          execute('storageattach', @uuid, '--storagectl', controller, '--port',
+                  port.to_s, '--device', device.to_s, '--type', type, '--medium',
+                  file, '--comment', comment)
+        end
+
         def clear_forwarded_ports
           retryable(on: Vagrant::Errors::VBoxManageError, tries: 3, sleep: 1) do
             args = []
@@ -38,6 +60,13 @@ module VagrantPlugins
           end
         end
 
+        # @param [String] source
+        # @param [String] destination
+        # @param [String] disk_format
+        def clone_disk(source, destination, disk_format, **opts)
+          execute("clonemedium", source, destination, '--format', disk_format)
+        end
+
         def clonevm(master_id, snapshot_name)
           machine_name = "temp_clone_#{(Time.now.to_f * 1000.0).to_i}_#{rand(100000)}"
           args = ["--register", "--name", machine_name]
@@ -49,6 +78,14 @@ module VagrantPlugins
           return get_machine_id(machine_name)
         end
 
+        # Removes a disk from the given virtual machine
+        #
+        # @param [String] disk_uuid or file path
+        # @param [Hash]   opts -  additional options
+        def close_medium(disk_uuid)
+          execute("closemedium", disk_uuid, '--delete')
+        end
+
         def create_dhcp_server(network, options)
           retryable(on: Vagrant::Errors::VBoxManageError, tries: 3, sleep: 1) do
             begin
@@ -65,6 +102,17 @@ module VagrantPlugins
           end
         end
 
+       # Creates a disk. Default format is VDI unless overridden
+       #
+       # @param [String] disk_file
+       # @param [Integer] disk_size - size in bytes
+       # @param [String] disk_format - format of disk, defaults to "VDI"
+        # @param [Hash]  opts -  additional options
+       def create_disk(disk_file, disk_size, disk_format="VDI", **opts)
+         execute("createmedium", '--filename', disk_file, '--sizebyte', disk_size.to_i.to_s, '--format', disk_format)
+       end
+
+
         def create_host_only_network(options)
           # Create the interface
           execute("hostonlyif", "create", retryable: true) =~ /^Interface '(.+?)' was successfully created$/
@@ -130,6 +178,33 @@ module VagrantPlugins
           end
         end
 
+        # Lists all attached harddisks from a given virtual machine. Additionally,
+        # this method adds a new key "Disk Name" based on the disks file path from "Location"
+        #
+        # @return [Array] hdds An array of hashes of harddrive info for a guest
+        def list_hdds
+          hdds = []
+          tmp_drive = {}
+          execute('list', 'hdds', retryable: true).split("\n").each do |line|
+            if line == "" # separator between disks
+              hdds << tmp_drive
+              tmp_drive = {}
+              next
+            end
+            parts = line.partition(":")
+            key = parts.first.strip
+            value = parts.last.strip
+            tmp_drive[key] = value
+
+            if key == "Location"
+              tmp_drive["Disk Name"] = File.basename(value, ".*")
+            end
+          end
+          hdds << tmp_drive unless tmp_drive.empty?
+
+          hdds
+        end
+
         def list_snapshots(machine_id)
           output = execute(
             "snapshot", machine_id, "list", "--machinereadable",
@@ -149,6 +224,21 @@ module VagrantPlugins
           raise
         end
 
+        # @param [String] port - port on device to attach disk to
+        # @param [String] device - device on controller for disk
+        # @param [Hash]   opts -  additional options
+        def remove_disk(port, device)
+          controller = "SATA Controller"
+          execute('storageattach', @uuid, '--storagectl', controller, '--port', port.to_s, '--device', device.to_s, '--medium', "none")
+        end
+
+        # @param [String] disk_file
+        # @param [Integer] disk_size in bytes
+        # @param [Hash]   opts -  additional options
+        def resize_disk(disk_file, disk_size, **opts)
+          execute("modifymedium", disk_file, '--resizebyte', disk_size.to_i.to_s)
+        end
+
         def restore_snapshot(machine_id, snapshot_name)
           # Start with 0%
           last = 0
@@ -295,6 +385,27 @@ module VagrantPlugins
           nil
         end
 
+        # Returns port and device for an attached disk given a disk uuid. Returns
+        # empty hash if disk is not attachd to guest
+        #
+        # @param [Hash] vm_info - A guests information from vboxmanage
+        # @param [String] disk_uuid - the UUID for the disk we are searching for
+        # @return [Hash] disk_info - Contains a device and port number
+        def get_port_and_device(disk_uuid)
+          vm_info = show_vm_info
+
+          disk = {}
+          disk_info_key = vm_info.key(disk_uuid)
+          return disk if !disk_info_key
+
+          disk_info = disk_info_key.split("-")
+
+          disk[:port] = disk_info[2]
+          disk[:device] = disk_info[3]
+
+          return disk
+        end
+
         def halt
           execute("controlvm", @uuid, "poweroff", retryable: true)
         end
@@ -370,7 +481,7 @@ module VagrantPlugins
             end
           end
 
-          return get_machine_id specified_name
+          return get_machine_id(specified_name)
         end
 
         def max_network_adapters
@@ -476,6 +587,11 @@ module VagrantPlugins
 
         def read_guest_ip(adapter_number)
           ip = read_guest_property("/VirtualBox/GuestInfo/Net/#{adapter_number}/V4/IP")
+          if ip.end_with?(".1")
+            @logger.warn("VBoxManage guest property returned: #{ip}. Result resembles IP of DHCP server and is being ignored.")
+            ip = nil
+          end
+
           if !valid_ip_address?(ip)
             raise Vagrant::Errors::VirtualBoxGuestPropertyNotFound,
               guest_property: "/VirtualBox/GuestInfo/Net/#{adapter_number}/V4/IP"
@@ -783,12 +899,36 @@ module VagrantPlugins
           return true
         end
 
+        # @param [VagrantPlugins::VirtualboxProvider::Driver] driver
+        # @param [String] defined_disk_path
+        # @return [String] destination - The cloned disk
+        def vmdk_to_vdi(defined_disk_path)
+          source = defined_disk_path
+          destination = File.join(File.dirname(source), File.basename(source, ".*")) + ".vdi"
+
+          clone_disk(source, destination, 'VDI')
+
+          destination
+        end
+
+        # @param [VagrantPlugins::VirtualboxProvider::Driver] driver
+        # @param [String] defined_disk_path
+        # @return [String] destination - The cloned disk
+        def vdi_to_vmdk(defined_disk_path)
+          source = defined_disk_path
+          destination = File.join(File.dirname(source), File.basename(source, ".*")) + ".vmdk"
+
+          clone_disk(source, destination, 'VMDK')
+
+          destination
+        end
+
         protected
 
         def valid_ip_address?(ip)
           # Filter out invalid IP addresses
           # GH-4658 VirtualBox can report an IP address of 0.0.0.0 for FreeBSD guests.
-          if ip == "0.0.0.0"
+          if ip == "0.0.0.0" || ip.nil?
             return false
           else
             return true
