@@ -38,6 +38,7 @@ describe Vagrant::Util::Downloader do
       before do
         allow(ui).to receive(:clear_line)
         allow(ui).to receive(:detail)
+        allow(ui).to receive(:rewriting).and_yield(ui)
       end
 
       after do
@@ -256,22 +257,37 @@ describe Vagrant::Util::Downloader do
           end
         end
       end
+
+      context "when extra download options specified" do
+        let(:options) { {:box_extra_download_options => ["--test", "arbitrary"]} }
+        subject { described_class.new(source, destination, options) }
+
+        it "inserts the extra download options" do
+          i = curl_options.index("--output")
+          curl_options.insert(i, "arbitrary")
+          curl_options.insert(i, "--test")
+          expect(Vagrant::Util::Subprocess).to receive(:execute).
+            with("curl", *curl_options).
+            and_return(subprocess_result)
+
+        expect(subject.download!).to be(true)
+        end
+      end
     end
   end
 
   describe "#head" do
     let(:curl_options) {
-      ["-q", "--fail", "--location", "--max-redirs", "10", "--verbose", "--user-agent", described_class::USER_AGENT, source, {}]
+      ["-q", "-I", "--fail", "--location", "--max-redirs", "10",
+       "--verbose", "--user-agent", described_class::USER_AGENT,
+       source, {}]
     }
 
     it "returns the output" do
       allow(subprocess_result).to receive(:stdout).and_return("foo")
 
-      options = curl_options.dup
-      options.unshift("-I")
-
       expect(Vagrant::Util::Subprocess).to receive(:execute).
-        with("curl", *options).and_return(subprocess_result)
+        with("curl", *curl_options).and_return(subprocess_result)
 
       expect(subject.head).to eq("foo")
     end
