@@ -49,5 +49,15 @@ describe Vagrant::Util::IsPortOpen do
     # best, really.
     expect(klass.is_port_open?("127.0.0.1", closed_port)).not_to be
   end
+
+  it "should handle connection refused" do
+    expect(TCPSocket).to receive(:new).with("0.0.0.0", closed_port).and_raise(Errno::ECONNREFUSED)
+    expect(klass.is_port_open?("0.0.0.0", closed_port)).to be(false)
+  end
+
+  it "should raise an error if cannot assign requested address" do
+    expect(TCPSocket).to receive(:new).with("0.0.0.0", open_port).and_raise(Errno::EADDRNOTAVAIL)
+    expect { klass.is_port_open?("0.0.0.0", open_port) }.to raise_error(Errno::EADDRNOTAVAIL)
+  end
 end
 
