@@ -280,7 +280,7 @@ describe Vagrant::Util::Platform do
       end
 
       it "should return false when path is within /mnt" do
-        expect(subject.wsl_path?("/mnt/c")).to be false
+        expect(subject.wsl_path?("/mnt/c")).to be(false)
       end
     end
 
@@ -534,4 +534,10 @@ EOF
       end
     end
   end
+
+  describe ".unix_windows_path" do
+    it "takes a windows path and returns a POSIX-like path" do
+      expect(subject.unix_windows_path("C:\\Temp\\Windows")).to eq("C:/Temp/Windows")
+    end
+  end
 end
